[Unit]
Description=ZSCAMs Service
After=network.target

[Service]
ExecStart={python_exec} -m zscams
Restart=always
RestartSec=5
User={user_to_run_as}
Environment="PYTHONPATH={pythonpath}"

[Install]
WantedBy=multi-user.target