#!/bin/sh
#
# PROVIDE: zscams
# REQUIRE: NETWORKING
# KEYWORD: shutdown

. /etc/rc.subr

name="zscams"
rcvar=zscams_enable

load_rc_config $name

: ${{zscams_enable:="NO"}}
: ${{zscams_user:="{user_to_run_as}"}}
: ${{zscams_python:="{python_exec}"}}

pidfile="/var/run/${{name}}.pid"
command="/usr/sbin/daemon"
command_args="-P ${{pidfile}} -r -t ${{name}} -o /var/log/${{name}}.log -u ${{zscams_user}} ${{zscams_python}} -m zscams"

run_rc_command "$1"
